module Dice exposing (..)

type alias Dice =
    { amount : Int
    , difficulty : Int
    , willpower : Bool
    , specialty : Bool
    }

new : Dice
new =
    { amount = 1
    , difficulty = 6
    , willpower = False
    , specialty = False
    }

changeAmountTo : Dice -> Int -> Dice
changeAmountTo dice amount =
    { dice | amount = amount }

changeDifficultyTo : Dice -> Int -> Dice
changeDifficultyTo dice difficulty =
    { dice 
    | difficulty =
        if difficulty > 10
        then 10
        else difficulty 
    }

switchWillpower : Dice -> Dice
switchWillpower dice =
    { dice | willpower = not dice.willpower }

switchSpecialty : Dice -> Dice
switchSpecialty dice =
    { dice | specialty = not dice.specialty }

successes : Dice -> List Int -> Int
successes dice roll =
    let
        special : Int
        special =
            if dice.specialty
            then
                roll
                |> List.filter ( \x -> x == 10 )
                |> List.length
            else 0
        plus : Int
        plus = roll
            |> List.filter ( \x -> x >= dice.difficulty )
            |> List.length
        minus : Int
        minus = roll
            |> List.filter ( \x -> x < dice.difficulty )
            |> List.filter ( \x -> x == 1 )
            |> List.length
        result = special + plus - minus
    in
        if dice.willpower
        then if result < 1
            then 1
            else result
        else result